from prometheus_client import start_http_server

from kvcommon_flask.vars import KVC_FLASK_METRICS_ENABLED as ENABLED
from kvcommon_flask.vars import KVC_FLASK_METRICS_PORT
from .metrics import JOB_EVENT

from .metrics import SECONDS_SERVER_REQUEST
from .metrics import incr_counter


def init_metrics():
    if ENABLED:
        start_http_server(KVC_FLASK_METRICS_PORT)

__all__ = [
    "ENABLED",
    "incr_counter",
    "init_metrics",
    "JOB_EVENT",
    "SECONDS_SERVER_REQUEST",
]



import logging
import typing as t


from flask_http_middleware import BaseHTTPMiddleware

from kvcommon.urls import urlparse_ignore_scheme
from kvcommon_flask import metrics
from kvcommon_flask.context import set_flask_context_local
from kvcommon import logger

LOG = logger.get_logger("kvc-flask")


def is_meta_url(url: str, prefix: str | None = None) -> bool:
    if (prefix and url.startswith(prefix)) or url.startswith("/healthz/"):
        return True
    return False


class KVCFlaskMiddleware(BaseHTTPMiddleware):
    _meta_prefix: str

    def __init__(self, meta_prefix: str | None):
        if meta_prefix:
            self._meta_prefix = meta_prefix
        super().__init__()

    def dispatch(self, request, call_next):
        with metrics.SECONDS_SERVER_REQUEST.labels().time():
            url_parts = urlparse_ignore_scheme(request.url, request.scheme)
            url_path: str = url_parts.path
            set_flask_context_local("url_parts", url_parts)

            is_meta: bool = is_meta_url(url_path, prefix=self._meta_prefix)
            set_flask_context_local("is_meta_request", is_meta)

            response = call_next(request)

        return response



from prometheus_client import Counter
from prometheus_client import Gauge
from prometheus_client import Histogram
from prometheus_client import Info
from prometheus_client import Summary


def incr_counter(metric: Counter | Gauge):
    from kvcommon_flask.vars import KVC_FLASK_METRICS_ENABLED

    if KVC_FLASK_METRICS_ENABLED:
        metric.inc()


def set_app_info(app_version: str):
    from kvcommon_flask.vars import KVC_FLASK_METRICS_ENABLED

    if KVC_FLASK_METRICS_ENABLED:
        APP_INFO.info(dict(version=app_version))


APP_INFO = Info("app", "Application info")


JOB_EVENT = Counter(
    "scheduled_job_event",
    "Gauge of scheduled job events by event enum",
    labelnames=[
        "job_id",
        "event",
    ],
)

# Total time spent from start to finish on a request
SECONDS_SERVER_REQUEST = Histogram(
    "seconds_server_request",
    "Time taken for server to handle request",
    labelnames=["path"],
)
